#include <acknex.h>
#include <default.c>

#define coll_axis_x 1
#define coll_axis_y 2
#define coll_axis_diag 3

typedef struct obj_side
{
	VECTOR start;
	VECTOR end;
	VECTOR dir;
	VECTOR axis;
	VECTOR center;
	int coll_axis_type;
	int length;
}
obj_side;

typedef struct tile
{
	int id;
	PANEL* image;
	int width;
	int height;
	int type;
	int c_type;
	int total_points;
	int points[8];
	obj_side* side[8];
	VECTOR p0;
	VECTOR p1;
	VECTOR p2;
	VECTOR p3;
	VECTOR p4;
	VECTOR p5;
	VECTOR p6;
	VECTOR p7;
	VECTOR p8;
	VECTOR center;
	VECTOR side_dot1;
	VECTOR side_dot2;
	VECTOR side_dir;
	VECTOR side_normal;
} tile;

#define c_type_triangle 0
#define c_type_square 1
#define c_type_circle_convex 2
#define c_type_player 3
#define c_type_circle_concave 4

#define tri_tile1 0
#define tri_tile2 1
#define tri_tile3 2
#define tri_tile4 3
#define tri_tile5 4
#define tri_tile6 5
#define tri_tile7 6
#define tri_tile8 7

#define square_tile1 8
#define square_tile2 9
#define square_tile3 10
#define square_tile4 11
#define square_tile5 12

#define cir_tile_convex1 13
#define cir_tile_convex2 14
#define cir_tile_convex3 15
#define cir_tile_convex4 16
#define cir_tile_convex5 17
#define cir_tile_convex6 18
#define cir_tile_convex7 19
#define cir_tile_convex8 20

#define cir_tile_concave1 21
#define cir_tile_concave2 22
#define cir_tile_concave3 23
#define cir_tile_concave4 24
#define cir_tile_concave5 25
#define cir_tile_concave6 26
#define cir_tile_concave7 27
#define cir_tile_concave8 28

PANEL* square_pan;
#define x_slots 24
#define y_slots 17

#define player_size 5
tile* player_tiles[player_size];

tile* grid_tiles[x_slots][y_slots];
VECTOR kb;
VECTOR kbd;
VECTOR hitside_normals;
var speed = 20;

void speedtg(){speed = 20 - speed;}

VECTOR* c_red			= {x=0;y=0;z=255;}
VECTOR* c_green		 	= {x=0;y=255;z=0;}
VECTOR* c_blue			= {x=255;y=0;z=0;}
VECTOR* c_yellow		= {x=0;y=255;z=255;}
VECTOR* c_purple		= {x=255;y=0;z=255;}

VECTOR* c_white			= {x=255;y=255;z=255;}

int i = 0;
int j = 0;
int k = 0;

STRING* tempstr = "1";
PANEL* pan_create2(char* _content, int _layer, STRING* _bitmap, int _pos_x, int _pos_y, long _flags)
{
	str_cpy(tempstr,"");
	if(_bitmap != 0)
	{
		str_cat(tempstr,"bmap=");
		str_cat(tempstr,_bitmap);
		str_cat(tempstr,";");
	}

	str_cat(tempstr,_content);

	PANEL* _pan = pan_create(tempstr,_layer);
	
	_pan.pos_x = _pos_x;
	_pan.pos_y = _pos_y;

	_pan.flags |= (_flags);
	return(_pan);
}


#define new(type) malloc(sizeof(type##))


var mybounce = 1;
var myfriction = 50;
var mygravity = 0;
var myspeed_mod = 1;

void bounceup(){mybounce=clamp(mybounce+=0.1,0,1);}
void bouncedown(){mybounce=clamp(mybounce-=0.1,0,1);}
void frictionup(){myfriction=clamp(myfriction++,1,100);}
void frictiondown(){myfriction=clamp(myfriction--,1,100);}

void gravityup(){mygravity=clamp(mygravity+=0.1,0,1);}
void gravitydown(){mygravity=clamp(mygravity-=0.1,0,1);}

void speedup(){myspeed_mod=clamp(myspeed_mod+=0.05,0.5,1.5);}
void speeddown(){myspeed_mod=clamp(myspeed_mod-=0.1,-1,1);}


void draw_line_vec2vec(VECTOR* _vec1,VECTOR* _vec2,VECTOR* _color,var _alpha)
{
	draw_line(_vec1.x, 0, _alpha);
	draw_line(vector(_vec1.x,_vec1.y,0),_color, _alpha);
	draw_line(vector(_vec2.x,_vec2.y,0),_color, _alpha);
}

int drag_active = 0;

void dragmee(var _button,PANEL* _pan)
{
	if(drag_active == 1){return;}
	drag_active = 1;

	var offset_x = _pan.pos_x + bmap_width(_pan.bmap) * 0.5 * _pan.scale_x - mouse_pos.x;
	var offset_y = _pan.pos_y + bmap_height(_pan.bmap) * 0.5 * _pan.scale_y - mouse_pos.y;
	var drag_limit = 0;
	while(mouse_left)
	{
		_pan.pos_x = mouse_pos.x - bmap_width(_pan.bmap) * 0.5 * _pan.scale_x + offset_x;
		_pan.pos_y = mouse_pos.y - bmap_height(_pan.bmap) * 0.5 * _pan.scale_y + offset_y;

		if(_pan.pos_x > (screen_size.x - (bmap_width(_pan.bmap) * _pan.scale_x) - drag_limit))
		{
			_pan.pos_x = screen_size.x - (bmap_width(_pan.bmap) * _pan.scale_x) - drag_limit;
		}

		if(_pan.pos_x < drag_limit)
		{
			_pan.pos_x = drag_limit;
		}

		if(_pan.pos_y > (screen_size.y - (bmap_height(_pan.bmap) * _pan.scale_y) - drag_limit))
		{
			_pan.pos_y = screen_size.y - (bmap_height(_pan.bmap) * _pan.scale_y) - drag_limit;
		}

		if(_pan.pos_y < drag_limit)
		{
			_pan.pos_y = drag_limit;
		}

		wait(1);
	}
	drag_active = 0;
}

void vec_for_pan_center(VECTOR* _vec,PANEL* _pan)
{
	_vec.x = _pan.pos_x + bmap_width(_pan.bmap) * 0.5 * _pan.scale_x;
	_vec.y = _pan.pos_y + bmap_height(_pan.bmap) * 0.5 * _pan.scale_y;
	_vec.z = 0;
}

void pan_center_for_vec(PANEL* _pan,VECTOR* _vec)
{
	_pan.pos_x = _vec.x - bmap_width(_pan.bmap) * 0.5 * _pan.scale_x;
	_pan.pos_y = _vec.y - bmap_height(_pan.bmap) * 0.5 * _pan.scale_y;
}


void vec_project(VECTOR* _return_vec,VECTOR* _src_vec,VECTOR* _tgt_vec)
{
	if(vec_length(_src_vec) == 0 || vec_length(_tgt_vec) == 0){return;}
	var dp = (_src_vec.x * _tgt_vec.x + _src_vec.y * _tgt_vec.y);

	_return_vec.x = ( dp / (_tgt_vec.x * _tgt_vec.x + _tgt_vec.y * _tgt_vec.y)) * _tgt_vec.x;
	_return_vec.y = ( dp / (_tgt_vec.x * _tgt_vec.x + _tgt_vec.y * _tgt_vec.y)) * _tgt_vec.y;
	_return_vec.z = 0; 
}

int lines_cross(VECTOR* resultvec,var x11,var y11,var x12,var y12,var x21,var y21,var x22,var y22)
{
	resultvec.x = 0;
	resultvec.y = 0;
	resultvec.z = 0;

	float x1 = x11, x2 = x12, x3 = x21, x4 = x22;
	float y1 = y11, y2 = y12, y3 = y21, y4 = y22;

	float denominator = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);
	if(denominator == 0){return(0);}
 
	float pre = (x1*y2 - y1*x2), post = (x3*y4 - y3*x4);
	float x = ( pre * (x3 - x4) - (x1 - x2) * post ) / denominator;
	float y = ( pre * (y3 - y4) - (y1 - y2) * post ) / denominator;
	 
	//    
	if ( x < minv(x1, x2) || x > maxv(x1, x2) || x < minv(x3, x4) || x > maxv(x3, x4) ){return(0);}
	if ( y < minv(y1, y2) || y > maxv(y1, y2) || y < minv(y3, y4) || y > maxv(y3, y4) ){return(0);}
	 
	resultvec.x = x;
	resultvec.y = y;
}



int lines_cross_outside(VECTOR* resultvec,var x11,var y11,var x12,var y12,var x21,var y21,var x22,var y22)
{
	resultvec.x = 0;
	resultvec.y = 0;
	resultvec.z = 0;

	float x1 = x11, x2 = x12, x3 = x21, x4 = x22;
	float y1 = y11, y2 = y12, y3 = y21, y4 = y22;

	float denominator = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);
	if(denominator == 0){return(0);}
 
	float pre = (x1*y2 - y1*x2), post = (x3*y4 - y3*x4);
	float x = ( pre * (x3 - x4) - (x1 - x2) * post ) / denominator;
	float y = ( pre * (y3 - y4) - (y1 - y2) * post ) / denominator;
	 
	////    
	//if ( x < minv(x1, x2) || x > maxv(x1, x2) || x < minv(x3, x4) || x > maxv(x3, x4) ){return(0);}
	//if ( y < minv(y1, y2) || y > maxv(y1, y2) || y < minv(y3, y4) || y > maxv(y3, y4) ){return(0);}
	 
	resultvec.x = x;
	resultvec.y = y;
}


int get_dot_on_side_from2(VECTOR* _result,VECTOR* _dot1,VECTOR* _dot2,var _side)
{
	VECTOR side_dot;
	var sdot;

	if(_dot1.x != _dot2.x)
	{
		if(_side == -1)
		{// 
			if(_dot1.x < _dot2.x){sdot = 1;}
			else{sdot = 2;}

			side_dot.x = minv(_dot1.x,_dot2.x);
		}
		else
		{//
			if(_dot1.x > _dot2.x){sdot = 1;}
			else{sdot = 2;}

			side_dot.x = maxv(_dot1.x,_dot2.x);
		}
		vec_set(_result,side_dot);
		return(sdot);
	}
	else
	{
		return(0);
	}
}

int get_dot_on_side_from4(VECTOR* _result,VECTOR* _dot1,VECTOR* _dot2,VECTOR* _dot3,VECTOR* _dot4,var _side)
{
	VECTOR side_dot1,side_dot2,side_dot;
	var sdot1,sdot2,sdot;

	if(_dot1.x != _dot2.x)
	{
		if(_side == -1)
		{// 
			if(_dot1.x < _dot2.x){sdot1 = 1;}//   
			else{sdot1 = 2;}//   

			side_dot1.x = minv(_dot1.x,_dot2.x);
			
			if(_dot3.x < _dot4.x){sdot2 = 3;}//   
			else{sdot2 = 4;}//   

			side_dot2.x = minv(_dot3.x,_dot4.x);
			
			if(side_dot1.x < side_dot2.x)
			{//        
				if(sdot1 == 1)
				{//   
					side_dot.x = _dot1.x;
					side_dot.y = _dot1.y;
				}
				else
				{//   
					side_dot.x = _dot2.x;
					side_dot.y = _dot2.y;
				}
				sdot = sdot1;
			}
			else
			{//   
				if(sdot2 == 3)
				{//   
					side_dot.x = _dot3.x;
					side_dot.y = _dot3.y;
				}
				else
				{//    
					side_dot.x = _dot4.x;
					side_dot.y = _dot4.y;
				}
				sdot = sdot2;
			}
		}
		else
		{//
			if(_dot1.x > _dot2.x){sdot1 = 1;}//   
			else{sdot1 = 2;}//   

			side_dot1.x = maxv(_dot1.x,_dot2.x);
			
			if(_dot3.x > _dot4.x){sdot2 = 3;}//   
			else{sdot2 = 4;}//   

			side_dot2.x = maxv(_dot3.x,_dot4.x);
			
			if(side_dot1.x > side_dot2.x)
			{//        
				if(sdot1 == 1)
				{//   
					side_dot.x = _dot1.x;
					side_dot.y = _dot1.y;
				}
				else
				{//   
					side_dot.x = _dot2.x;
					side_dot.y = _dot2.y;
				}
				sdot = sdot1;
			}
			else
			{//   
				if(sdot2 == 3)
				{//   
					side_dot.x = _dot3.x;
					side_dot.y = _dot3.y;
				}
				else
				{//    
					side_dot.x = _dot4.x;
					side_dot.y = _dot4.y;
				}
				sdot = sdot2;
			}
		}
	
		vec_set(_result,side_dot);
		return(sdot);
	}
	else
	{
		return(0);
	}
}

int proj_intersect_vert(VECTOR* _dot1,VECTOR* _dot2,tile* tri,int _p_side)
{
	//draw_line_vec2vec(tri.p0,tri.p1,c_white,100);
	var square_center = _dot1.y + (_dot2.y - _dot1.y)*0.5;
	var square_proj_size = abs(_dot1.y-_dot2.y)*0.5;
	var tri_proj_size = abs(tri.side[_p_side].length)*0.5;
	var dist = square_center - tri.side[_p_side].center.y;
	var knockback_dir = maxv(square_proj_size+tri_proj_size-abs(dist),0);
	if(dist<=0){knockback_dir *= -1;}

	//draw_line_vec2vec(vector(square_center,tri.side[_p_side].center.y-3,0),vector(square_center+square_proj_size,tri.side[_p_side].center.y-3,0),c_blue,100);
	
	if(knockback_dir!=0)
	{
		draw_line_vec2vec(vector(tri.side[_p_side].center.x+3,square_center,0),vector(tri.side[_p_side].center.x+3,square_center+square_proj_size,0),c_blue,100);
		draw_line_vec2vec(vector(tri.side[_p_side].center.x+3,tri.side[_p_side].center.y,0),vector(tri.side[_p_side].center.x+3,tri.side[_p_side].center.y+tri_proj_size,0),c_red,100);
		draw_line_vec2vec(vector(tri.side[_p_side].center.x+4,tri.side[_p_side].center.y,0),vector(tri.side[_p_side].center.x+4,tri.side[_p_side].center.y+knockback_dir,0),c_green,100);
	}

	//draw_line_vec2vec(_dot1,_dot2,c_white,100);
	return(knockback_dir);
}

int proj_intersect_hori(VECTOR* _dot1,VECTOR* _dot2,tile* tri,int _p_side)
{
	//draw_line_vec2vec(tri.p0,tri.p1,c_white,100);
	var square_center = _dot1.x + (_dot2.x - _dot1.x)*0.5;
	var square_proj_size = abs(_dot1.x-_dot2.x)*0.5;
	var tri_proj_size = abs(tri.side[_p_side].length)*0.5;
	var dist = square_center - tri.side[_p_side].center.x;
	var knockback_dir = maxv(square_proj_size+tri_proj_size-abs(dist),0);
	if(dist<=0){knockback_dir *= -1;}

	//draw_line_vec2vec(vector(square_center,tri.side[_p_side].center.y-3,0),vector(square_center+square_proj_size,tri.side[_p_side].center.y-3,0),c_blue,100);
	
	if(knockback_dir!=0)
	{
		draw_line_vec2vec(vector(square_center,tri.side[_p_side].center.y-3,0),vector(square_center+square_proj_size,tri.side[_p_side].center.y-3,0),c_blue,100);
		draw_line_vec2vec(vector(tri.side[_p_side].center.x,tri.side[_p_side].center.y-3,0),vector(tri.side[_p_side].center.x+tri_proj_size,tri.side[_p_side].center.y-3,0),c_red,100);
		draw_line_vec2vec(vector(tri.side[_p_side].center.x,tri.side[_p_side].center.y-4,0),vector(tri.side[_p_side].center.x+knockback_dir,tri.side[_p_side].center.y-4,0),c_green,100);
	}

	//draw_line_vec2vec(_dot1,_dot2,c_white,100);
	return(knockback_dir);
}

int proj_intersect_diagonal(VECTOR* _dot1,VECTOR* _dot2,VECTOR* _dot3,VECTOR* _dot4,tile* tri,int _p_side,int _a_side,VECTOR* _result)
{
	VECTOR square_center,square_intersect_dir,square_intersect_dot;
	VECTOR tempvec,tempdir;
	VECTOR tri_proj_center;
	VECTOR square_proj1,square_proj2,tri_proj;
	VECTOR square_proj_dot1,square_proj_dot2,tri_proj_dot1,tri_proj_dot2;

	var knockback_invert;
	// 
	var square_xhside = (_dot2.x - _dot1.x)*0.5;
	var square_yhside = (_dot4.y - _dot1.y)*0.5;
	vec_set(square_center,vector(_dot1.x + square_xhside,_dot1.y + square_yhside,0));

	//   ,    
	vec_set(square_intersect_dir,vector(square_center.x + tri.side[_a_side].dir.x,square_center.y + tri.side[_a_side].dir.y,0));

	//        
	lines_cross_outside(square_intersect_dot,
		square_center.x,square_center.y,square_intersect_dir.x,square_intersect_dir.y,
		tri.side[_a_side].center.x,tri.side[_a_side].center.y,tri.side[_a_side].center.x+tri.side[_a_side].axis.x,tri.side[_a_side].center.y+tri.side[_a_side].axis.y);

	//      
	var	xside = 1 - 2 * (square_intersect_dot.x < square_center.x);//     ?
	var	yside = 1 - 2 * (square_intersect_dot.y < square_center.y);//     ?

	//  
	vec_set(square_proj_dot1,vector(square_center.x,square_center.y+square_yhside*yside,0));
	vec_set(square_proj_dot2,vector(square_center.x+square_xhside*xside,square_center.y,0));

	//    
	vec_diff(tempdir,square_proj_dot1,square_center);
	vec_project(square_proj1,tempdir,tri.side[_a_side].axis);

	vec_diff(tempdir,square_proj_dot2,square_center);
	vec_project(square_proj2,tempdir,tri.side[_a_side].axis);

	//       
	var len = vec_length(square_proj1) + vec_length(square_proj2);
	vec_normalize(square_proj1,len);
	vec_normalize(square_proj2,len);
	
	//  
	vec_diff(tempdir,tri.side[_p_side].end,tri.side[_p_side].start);
	vec_project(tri_proj,tempdir,tri.side[_a_side].axis);

	vec_set(tri_proj_center,vector(tri.side[_a_side].center.x+tri_proj.x*0.5,tri.side[_a_side].center.y+tri_proj.y*0.5,0));

	vec_set(tri_proj_dot1,vector(tri.side[_a_side].center.x+tri_proj.x,tri.side[_a_side].center.y+tri_proj.y,0));
	vec_set(tri_proj_dot2,tri.side[_a_side].center);

	vec_set(square_proj_dot1,vector(square_intersect_dot.x+square_proj1.x,square_intersect_dot.y+square_proj1.y,0));
	vec_set(square_proj_dot2,vector(square_intersect_dot.x+square_proj2.x,square_intersect_dot.y+square_proj2.y,0));
	

	VECTOR left_dot,right_dot;

	get_dot_on_side_from4(left_dot,square_proj_dot1.x,square_proj_dot2.x,tri_proj_dot1.x,tri_proj_dot2.x,-1);
	get_dot_on_side_from4(right_dot,square_proj_dot1.x,square_proj_dot2.x,tri_proj_dot1.x,tri_proj_dot2.x,1);
	
	knockback_invert = get_dot_on_side_from2(tempvec,tri_proj_center,square_intersect_dot,-1);

	vec_diff(tempdir,left_dot,right_dot);
	var lenlen = len*2 + vec_length(tri_proj) - vec_length(tempdir);

	// 
	vec_set(tempvec,tri.side[_a_side].axis);
	vec_normalize(tempvec,maxv(lenlen,0));
	
	if(knockback_invert == 1){vec_inverse(tempvec);}
	
	vec_set(_result,tempvec);

	//draw_line_vec2vec(tri.side[_a_side].center,mouse_pos.x,c_blue,50);

	draw_line_vec2vec(vector(tri_proj_center.x,tri_proj_center.y-10,0),vector(tri_proj_center.x+tempvec.x,tri_proj_center.y-10+tempvec.y,0),c_green,50);

	if(vec_length(tempvec)!=0)
	{
		draw_line_vec2vec(square_intersect_dot,square_center,c_blue,50);
		draw_line_vec2vec(square_intersect_dot,vector(square_intersect_dot.x+square_proj1.x,square_intersect_dot.y+square_proj1.y,0),c_blue,50);
		draw_line_vec2vec(square_intersect_dot,vector(square_intersect_dot.x+square_proj2.x,square_intersect_dot.y+square_proj2.y,0),c_blue,50);

		draw_line_vec2vec(tri.side[_a_side].center,vector(tri.side[_a_side].center.x+tri_proj.x,tri.side[_a_side].center.y+tri_proj.y,0),c_red,50);
	}
	return(vec_length(tempvec));
}


int proj_intersect_circle_convex(VECTOR* _dot1,VECTOR* _dot2,VECTOR* _dot3,VECTOR* _dot4,tile* cir,VECTOR* _result)
{	
	VECTOR square_center;
	VECTOR tempvec,tempdir;
	VECTOR closest_dot,axis;

	VECTOR square_proj,cir_proj;
	VECTOR square_proj_dot1,square_proj_dot2;

	//draw_line_vec2vec(cir.p0,cir.p1,c_white,100);
	//draw_line_vec2vec(cir.p1,cir.p2,c_white,100);
	//draw_line_vec2vec(cir.p2,cir.p0,c_white,100);

	// 
	vec_set(square_center,vector(_dot1.x + (_dot2.x - _dot1.x)*0.5,_dot1.y + (_dot4.y - _dot1.y)*0.5,0));

	//   
	vec_diff(tempvec,square_center,cir.p0);
	if(vec_length(tempvec) < cir.side[i].length){return(-777);}

	//   
	if(_dot1.x < cir.p0.x || _dot2.x < cir.p0.x || _dot3.x < cir.p0.x || _dot4.x < cir.p0.x){if(cir.type == cir_tile_convex1 || cir.type == cir_tile_convex4){return(-777);}}
	//
	if(_dot1.x > cir.p0.x || _dot2.x > cir.p0.x || _dot3.x > cir.p0.x || _dot4.x > cir.p0.x){if(cir.type == cir_tile_convex2 || cir.type == cir_tile_convex3){return(-777);}}
	//
	if(_dot1.y < cir.p0.y || _dot2.y < cir.p0.y || _dot3.y < cir.p0.y || _dot4.y < cir.p0.y){if(cir.type == cir_tile_convex1 || cir.type == cir_tile_convex2){return(-777);}}
	//
	if(_dot1.y > cir.p0.y || _dot2.y > cir.p0.y || _dot3.y > cir.p0.y || _dot4.y > cir.p0.y){if(cir.type == cir_tile_convex3 || cir.type == cir_tile_convex4){return(-777);}}

	// 
	var len,cd;
	vec_diff(tempvec,_dot1,cir.p0);
	len = vec_length(tempvec);
	vec_set(closest_dot,_dot1);
	vec_set(axis,tempvec);
	cd = 1;

	vec_diff(tempvec,_dot2,cir.p0);
	if(len>vec_length(tempvec))
	{
		len = vec_length(tempvec);
		vec_set(closest_dot,_dot2);
		vec_set(axis,tempvec);
		cd = 2;
	}
	
	vec_diff(tempvec,_dot3,cir.p0);
	if(len>vec_length(tempvec))
	{
		len = vec_length(tempvec);
		vec_set(closest_dot,_dot3);
		vec_set(axis,tempvec);
		cd = 3;
	}

	vec_diff(tempvec,_dot4,cir.p0);
	if(len>vec_length(tempvec))
	{
		len = vec_length(tempvec);
		vec_set(closest_dot,_dot4);
		vec_set(axis,tempvec);
		cd = 4;
	}

	if(cd == 1){vec_set(square_proj_dot1,_dot1);vec_set(square_proj_dot2,_dot3);}
	if(cd == 2){vec_set(square_proj_dot1,_dot2);vec_set(square_proj_dot2,_dot4);}
	if(cd == 3){vec_set(square_proj_dot1,_dot3);vec_set(square_proj_dot2,_dot1);}
	if(cd == 4){vec_set(square_proj_dot1,_dot4);vec_set(square_proj_dot2,_dot2);}

	//    
	vec_diff(tempdir,square_proj_dot2,square_proj_dot1);
	vec_project(square_proj,tempdir,axis);

	vec_set(cir_proj,axis);
	vec_normalize(cir_proj,cir.side[i].length);
	
	vec_diff(tempdir,cir.p0,vector(closest_dot.x+square_proj.x,closest_dot.y+square_proj.y,0));

	var lenlen = vec_length(square_proj) + vec_length(cir_proj) - vec_length(tempdir);

	// 
	vec_set(tempvec,axis);
	vec_normalize(tempvec,maxv(lenlen,0));
	vec_set(_result,tempvec);

	draw_line_vec2vec(cir.p0,vector(cir.p0.x+cir_proj.x,cir.p0.y+cir_proj.y,0),c_white,100);
	draw_line_vec2vec(cir.p0,vector(cir.p0.x+tempvec.x,cir.p0.y+tempvec.y,0),c_green,100);

	return(vec_length(tempvec));

}

int proj_intersect_circle_concave(VECTOR* _dot1,VECTOR* _dot2,VECTOR* _dot3,VECTOR* _dot4,tile* cir,VECTOR* _result)
{
	//VECTOR square_center;
	VECTOR tempdir;
	VECTOR closest_dot;
	VECTOR axis;
	VECTOR dot_to_edge,p3_to_dot;
	//VECTOR square_proj,cir_proj;
	//VECTOR square_proj_dot1,square_proj_dot2;
	VECTOR dot_at_edge;

	// 
	//vec_set(square_center,vector(_dot1.x + (_dot2.x - _dot1.x)*0.5,_dot1.y + (_dot4.y - _dot1.y)*0.5,0));

	////   
	//vec_diff(tempvec,square_center,cir.p0);
	//if(vec_length(tempvec) < cir.side[i].length){return(-777);}

	//   
	if(_dot1.x < cir.p0.x || _dot2.x < cir.p0.x || _dot3.x < cir.p0.x || _dot4.x < cir.p0.x){if(cir.type == cir_tile_concave1 || cir.type == cir_tile_concave4){return(-777);}}
	//
	if(_dot1.x > cir.p0.x || _dot2.x > cir.p0.x || _dot3.x > cir.p0.x || _dot4.x > cir.p0.x){if(cir.type == cir_tile_concave2 || cir.type == cir_tile_concave3){return(-777);}}
	//
	if(_dot1.y < cir.p0.y || _dot2.y < cir.p0.y || _dot3.y < cir.p0.y || _dot4.y < cir.p0.y){if(cir.type == cir_tile_concave1 || cir.type == cir_tile_concave2){return(-777);}}
	//
	if(_dot1.y > cir.p0.y || _dot2.y > cir.p0.y || _dot3.y > cir.p0.y || _dot4.y > cir.p0.y){if(cir.type == cir_tile_concave3 || cir.type == cir_tile_concave4){return(-777);}}

	//vec_set(closest_dot,_dot1);
	//vec_set(axis,tempvec);

	// 
	var len,cd;

	len = vec_dist(_dot1,cir.p0);
	vec_set(closest_dot,_dot1);
	//vec_diff(axis,closest_dot,cir.p3);
	cd = 1;

	if(len>vec_dist(_dot2,cir.p0))
	{
		len = vec_dist(_dot2,cir.p0);
		vec_set(closest_dot,_dot2);
		//vec_diff(axis,closest_dot,cir.p3);
		cd = 2;
	}

	if(len>vec_dist(_dot3,cir.p0))
	{
		len = vec_dist(_dot3,cir.p0);
		vec_set(closest_dot,_dot3);
		//vec_diff(axis,closest_dot,cir.p3);
		cd = 3;
	}

	if(len>vec_dist(_dot4,cir.p0))
	{
		len = vec_dist(_dot4,cir.p0);
		vec_set(closest_dot,_dot4);
		//vec_diff(axis,closest_dot,cir.p3);
		cd = 4;
	}

	//   
	if(closest_dot.x < cir.p0.x || closest_dot.y < cir.p0.y || closest_dot.x > cir.p3.x || closest_dot.y > cir.p3.y){if(cir.type == cir_tile_concave1){return(-888);}}
	//
	if(closest_dot.x < cir.p2.x || closest_dot.y < cir.p2.y || closest_dot.x > cir.p1.x || closest_dot.y > cir.p1.y){if(cir.type == cir_tile_concave2){return(-888);}}
	//
	if(closest_dot.x < cir.p3.x || closest_dot.y < cir.p3.y || closest_dot.x > cir.p0.x || closest_dot.y > cir.p0.y){if(cir.type == cir_tile_concave3){return(-888);}}
	//
	if(closest_dot.x < cir.p1.x || closest_dot.y < cir.p1.y || closest_dot.x > cir.p2.x || closest_dot.y > cir.p2.y){if(cir.type == cir_tile_concave4){return(-888);}}

	//if(cd == 1){vec_set(square_proj_dot1,_dot1);vec_set(square_proj_dot2,_dot3);}
	//if(cd == 2){vec_set(square_proj_dot1,_dot2);vec_set(square_proj_dot2,_dot4);}
	//if(cd == 3){vec_set(square_proj_dot1,_dot3);vec_set(square_proj_dot2,_dot1);}
	//if(cd == 4){vec_set(square_proj_dot1,_dot4);vec_set(square_proj_dot2,_dot2);}

	//vec_diff(tempdir,square_proj_dot2,square_proj_dot1);
	//vec_project(square_proj,tempdir,axis);

	//3  
	vec_diff(tempdir,closest_dot.x,cir.p3.x);
	vec_set(p3_to_dot,tempdir);

	//  
	vec_normalize(tempdir,cir.side[i].length);
	vec_set(dot_at_edge,vector(cir.p3.x+tempdir.x,cir.p3.y+tempdir.y,0));

	//       
	vec_diff(dot_to_edge,closest_dot,dot_at_edge);
	draw_line_vec2vec(dot_at_edge.x,vector(dot_at_edge.x+dot_to_edge.x,dot_at_edge.y+dot_to_edge.y,0),c_green,100);

	if(vec_length(dot_to_edge) + vec_length(p3_to_dot) - cir.side[i].length < 0){return(-999);}
	else
	{
		vec_inverse(dot_to_edge);
		vec_set(_result,dot_to_edge);
		return(1);
	}
}

tile* create_tile_triangle(STRING* _bmap,int _x_slot,int _y_slot,int _layer,int _type)
{
	tile* tri;
	tri = new(tile);
	int i;
	tri.image = pan_create2("",_layer,_bmap,_x_slot*40,_y_slot*40,VISIBLE | TRANSLUCENT | LIGHT);
	vec_set(tri.image.blue,c_red);
	//tri.image.scale_x=_scale_x;
	//tri.image.scale_y=_scale_y;
	tri.image.alpha = 10;

	//tri.id = total_tiles++;

	tri.total_points = 3;
	//tri.pri_dot = 0;

	//p0  
	//p0-p1  
	//p0-p2 
	if(_type == tri_tile1)//topleft
	{
		vec_set(tri.p0,vector(tri.image.pos_x,tri.image.pos_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y,0));

	}
	if(_type == tri_tile2)//topright
	{
		vec_set(tri.p0,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x,tri.image.pos_y,0));
	}
	if(_type == tri_tile3)//downright
	{
		vec_set(tri.p0,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
	}
	if(_type == tri_tile4)//downleft
	{
		vec_set(tri.p0,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x,tri.image.pos_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
	}


	if(_type == tri_tile5)//topleft
	{
		vec_set(tri.p0,vector(tri.image.pos_x,tri.image.pos_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*0.5*tri.image.scale_x,tri.image.pos_y,0));

	}
	if(_type == tri_tile6)//topright
	{
		vec_set(tri.p0,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*0.5*tri.image.scale_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x,tri.image.pos_y,0));
	}
	if(_type == tri_tile7)//downright
	{
		vec_set(tri.p0,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*0.5*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
	}
	if(_type == tri_tile8)//downleft
	{
		vec_set(tri.p0,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
		vec_set(tri.p1,vector(tri.image.pos_x,tri.image.pos_y+bmap_height(tri.image.bmap)*0.5*tri.image.scale_y,0));
		vec_set(tri.p2,vector(tri.image.pos_x+bmap_width(tri.image.bmap)*tri.image.scale_x,tri.image.pos_y+bmap_height(tri.image.bmap)*tri.image.scale_y,0));
	}

	for(i=0;i<tri.total_points;i++)
	{
		tri.side[i] = new(obj_side);
		if(i == 0)
		{
			vec_set(tri.side[i].start,tri.p0);
			vec_set(tri.side[i].end,tri.p1);
		}
		if(i == 1)
		{
			vec_set(tri.side[i].start,tri.p1);
			vec_set(tri.side[i].end,tri.p2);
		}

		if(i == 2)
		{
			vec_set(tri.side[i].start,tri.p2);
			vec_set(tri.side[i].end,tri.p0);
		}

		vec_diff(tri.side[i].dir,tri.side[i].end,tri.side[i].start);
		vec_set(tri.side[i].center,vector(tri.side[i].start.x+tri.side[i].dir.x*0.5,tri.side[i].start.y+tri.side[i].dir.y*0.5,0));

		//  ,   
	if(_type == tri_tile1||_type == tri_tile2||_type == tri_tile5||_type == tri_tile6)
	{
		tri.side[i].axis.x = tri.side[i].dir.y;
		tri.side[i].axis.y = tri.side[i].dir.x*-1;
	}

	if(_type == tri_tile3||_type == tri_tile4||_type == tri_tile7||_type == tri_tile8)
	{
		tri.side[i].axis.x = tri.side[i].dir.y*-1;
		tri.side[i].axis.y = tri.side[i].dir.x;
	}

		tri.side[i].axis.z = 0;


		tri.side[i].coll_axis_type = 0;

		if(tri.side[i].start.x == tri.side[i].end.x)
		{//      
			tri.side[i].coll_axis_type = coll_axis_x;
		}

		if(tri.side[i].start.y == tri.side[i].end.y)
		{//      
			tri.side[i].coll_axis_type = coll_axis_y;
		}

		if(tri.side[i].start.x != tri.side[i].end.x && tri.side[i].start.y != tri.side[i].end.y)
		{//  
			tri.side[i].coll_axis_type = coll_axis_diag;
		}

		if(tri.side[i].start.x == tri.side[i].end.x && tri.side[i].start.y == tri.side[i].end.y)
		{// 

		}

		tri.side[i].length = vec_length(tri.side[i].dir);
	}

	tri.width = bmap_width(tri.image.bmap)*tri.image.scale_x;
	tri.height = bmap_height(tri.image.bmap)*tri.image.scale_y;
	tri.type = _type;

	vec_for_pan_center(tri.center,tri.image);

	tri.c_type = c_type_triangle;
	if(_layer != 77){grid_tiles[_x_slot][_y_slot] = tri;}
	return(tri);
}

tile* create_tile_square(STRING* _bmap,int _x_slot,int _y_slot,int _layer,int _type)
{
	tile* square;
	square = new(tile);
	int i;
	square.image = pan_create2("",_layer,_bmap,_x_slot*40,_y_slot*40,VISIBLE | TRANSLUCENT | LIGHT);
	vec_set(square.image.blue,c_red);
	square.image.alpha = 10;

	//square.id = total_tiles++;

	square.total_points = 4;
	//square.pri_dot = 0;

	if(_type == square_tile1)//left
	{
		vec_set(square.p0,vector(square.image.pos_x,square.image.pos_y,0));
		vec_set(square.p1,vector(square.image.pos_x+bmap_width(square.image.bmap)*0.5*square.image.scale_x,square.image.pos_y,0));
		vec_set(square.p2,vector(square.image.pos_x+bmap_width(square.image.bmap)*0.5*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		vec_set(square.p3,vector(square.image.pos_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		square.width = bmap_width(square.image.bmap)*0.5*square.image.scale_x;
		square.height = bmap_height(square.image.bmap)*square.image.scale_y;
	}

	if(_type == square_tile2)//top
	{
		vec_set(square.p0,vector(square.image.pos_x,square.image.pos_y,0));
		vec_set(square.p1,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y,0));
		vec_set(square.p2,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*0.5*square.image.scale_y,0));
		vec_set(square.p3,vector(square.image.pos_x,square.image.pos_y+bmap_height(square.image.bmap)*0.5*square.image.scale_y,0));
		square.width = bmap_width(square.image.bmap)*square.image.scale_x;
		square.height = bmap_height(square.image.bmap)*0.5*square.image.scale_y;
	}

	if(_type == square_tile3)//right
	{
		vec_set(square.p0,vector(square.image.pos_x+bmap_width(square.image.bmap)*0.5*square.image.scale_x,square.image.pos_y,0));
		vec_set(square.p1,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y,0));
		vec_set(square.p2,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		vec_set(square.p3,vector(square.image.pos_x+bmap_width(square.image.bmap)*0.5*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		square.width = bmap_width(square.image.bmap)*0.5*square.image.scale_x;
		square.height = bmap_height(square.image.bmap)*square.image.scale_y;
	}

	if(_type == square_tile4)//down
	{
		vec_set(square.p0,vector(square.image.pos_x,square.image.pos_y+bmap_height(square.image.bmap)*0.5*square.image.scale_y,0));
		vec_set(square.p1,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*0.5*square.image.scale_y,0));
		vec_set(square.p2,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		vec_set(square.p3,vector(square.image.pos_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		square.width = bmap_width(square.image.bmap)*square.image.scale_x;
		square.height = bmap_height(square.image.bmap)*0.5*square.image.scale_y;
	}

	if(_type == square_tile5)//full
	{
		vec_set(square.p0,vector(square.image.pos_x,square.image.pos_y,0));
		vec_set(square.p1,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y,0));
		vec_set(square.p2,vector(square.image.pos_x+bmap_width(square.image.bmap)*square.image.scale_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		vec_set(square.p3,vector(square.image.pos_x,square.image.pos_y+bmap_height(square.image.bmap)*square.image.scale_y,0));
		square.width = bmap_width(square.image.bmap)*square.image.scale_x;
		square.height = bmap_height(square.image.bmap)*square.image.scale_y;
	}
	
	vec_set(square.center,vector(square.p0.x+square.width*0.5,square.p0.y+square.height*0.5,0));

	for(i=0;i<square.total_points;i++)
	{
		square.side[i] = new(obj_side);
		if(i == 0)
		{
			vec_set(square.side[i].start,square.p0);
			vec_set(square.side[i].end,square.p1);
		}
		if(i == 1)
		{
			vec_set(square.side[i].start,square.p1);
			vec_set(square.side[i].end,square.p2);
		}

		if(i == 2)
		{
			vec_set(square.side[i].start,square.p2);
			vec_set(square.side[i].end,square.p3);
		}

		if(i == 3)
		{
			vec_set(square.side[i].start,square.p3);
			vec_set(square.side[i].end,square.p0);
		}

		vec_diff(square.side[i].dir,square.side[i].end,square.side[i].start);
		vec_set(square.side[i].center,vector(square.side[i].start.x+square.side[i].dir.x*0.5,square.side[i].start.y+square.side[i].dir.y*0.5,0));

		if(square.side[i].start.x == square.side[i].end.x)
		{//      
			square.side[i].coll_axis_type = coll_axis_x;
		}

		if(square.side[i].start.y == square.side[i].end.y)
		{//      
			square.side[i].coll_axis_type = coll_axis_y;
		}

		if(square.side[i].start.x == square.side[i].end.x && square.side[i].start.y == square.side[i].end.y)
		{// 

		}

		square.side[i].length = vec_length(square.side[i].dir);
	}

	square.type = _type;
	square.c_type = c_type_square;
	if(_layer != 77){grid_tiles[_x_slot][_y_slot] = square;}
	return(square);
}

tile* create_tile_circle_convex(STRING* _bmap,int _x_slot,int _y_slot,int _layer,int _type)
{
	tile* cir;
	cir = new(tile);
	int i;
	cir.image = pan_create2("",_layer,_bmap,_x_slot*40,_y_slot*40,VISIBLE | TRANSLUCENT | LIGHT);
	vec_set(cir.image.blue,c_red);
	cir.image.alpha = 10;

	cir.total_points = 3;
	//cir.pri_dot = 0;

	//p0  
	if(_type == cir_tile_convex1)//topleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));

	}
	if(_type == cir_tile_convex2)//topright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y,0));
	}
	if(_type == cir_tile_convex3)//downright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}
	if(_type == cir_tile_convex4)//downleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}


	if(_type == cir_tile_convex5)//topleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*0.5*cir.image.scale_x,cir.image.pos_y,0));

	}
	if(_type == cir_tile_convex6)//topright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*0.5*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y,0));
	}
	if(_type == cir_tile_convex7)//downright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*0.5*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}
	if(_type == cir_tile_convex8)//downleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*0.5*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}

	for(i=0;i<cir.total_points;i++)
	{
		cir.side[i] = new(obj_side);
		if(i == 0)
		{
			vec_set(cir.side[i].start,cir.p0);
			vec_set(cir.side[i].end,cir.p1);
		}
		if(i == 1)
		{
			vec_set(cir.side[i].start,cir.p1);
			vec_set(cir.side[i].end,cir.p2);
		}

		if(i == 2)
		{
			vec_set(cir.side[i].start,cir.p2);
			vec_set(cir.side[i].end,cir.p0);
		}

		vec_diff(cir.side[i].dir,cir.side[i].end,cir.side[i].start);
		vec_set(cir.side[i].center,vector(cir.side[i].start.x+cir.side[i].dir.x*0.5,cir.side[i].start.y+cir.side[i].dir.y*0.5,0));

		//  ,   
	if(_type == cir_tile_convex1||_type == cir_tile_convex2||_type == cir_tile_convex5||_type == cir_tile_convex6)
	{
		cir.side[i].axis.x = cir.side[i].dir.y;
		cir.side[i].axis.y = cir.side[i].dir.x*-1;
	}

	if(_type == cir_tile_convex3||_type == cir_tile_convex4||_type == cir_tile_convex7||_type == cir_tile_convex8)
	{
		cir.side[i].axis.x = cir.side[i].dir.y*-1;
		cir.side[i].axis.y = cir.side[i].dir.x;
	}

		cir.side[i].axis.z = 0;


		cir.side[i].coll_axis_type = 0;

		if(cir.side[i].start.x == cir.side[i].end.x)
		{//      
			cir.side[i].coll_axis_type = coll_axis_x;
		}

		if(cir.side[i].start.y == cir.side[i].end.y)
		{//      
			cir.side[i].coll_axis_type = coll_axis_y;
		}

		if(cir.side[i].start.x != cir.side[i].end.x && cir.side[i].start.y != cir.side[i].end.y)
		{//  
			cir.side[i].coll_axis_type = coll_axis_diag;
		}

		if(cir.side[i].start.x == cir.side[i].end.x && cir.side[i].start.y == cir.side[i].end.y)
		{// 

		}

		cir.side[i].length = vec_length(cir.side[i].dir);
	}

	cir.width = bmap_width(cir.image.bmap)*cir.image.scale_x;
	cir.height = bmap_height(cir.image.bmap)*cir.image.scale_y;
	cir.type = _type;

	vec_for_pan_center(cir.center,cir.image);

	cir.c_type = c_type_circle_convex;
	if(_layer != 77){grid_tiles[_x_slot][_y_slot] = cir;}
	return(cir);
}
tile* create_tile_circle_concave(STRING* _bmap,int _x_slot,int _y_slot,int _layer,int _type)
{
	tile* cir;
	cir = new(tile);
	int i;
	cir.image = pan_create2("",_layer,_bmap,_x_slot*40,_y_slot*40,VISIBLE | TRANSLUCENT | LIGHT);
	vec_set(cir.image.blue,c_red);
	cir.image.alpha = 10;

	cir.total_points = 3;
	//cir.pri_dot = 0;

	//p0  
	if(_type == cir_tile_concave1)//topleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p3,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}
	if(_type == cir_tile_concave2)//topright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p3,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	}

	if(_type == cir_tile_concave3)//downright
	{
		vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p3,vector(cir.image.pos_x,cir.image.pos_y,0));
	}

	if(_type == cir_tile_concave4)//downleft
	{
		vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y,0));
		vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
		vec_set(cir.p3,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
	}


	//if(_type == cir_tile_convex5)//topleft
	//{
	//	vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y,0));
	//	vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	//	vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*0.5*cir.image.scale_x,cir.image.pos_y,0));

	//}
	//if(_type == cir_tile_convex6)//topright
	//{
	//	vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
	//	vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*0.5*cir.image.scale_y,0));
	//	vec_set(cir.p2,vector(cir.image.pos_x,cir.image.pos_y,0));
	//}
	//if(_type == cir_tile_convex7)//downright
	//{
	//	vec_set(cir.p0,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	//	vec_set(cir.p1,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y,0));
	//	vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*0.5*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	//}
	//if(_type == cir_tile_convex8)//downleft
	//{
	//	vec_set(cir.p0,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	//	vec_set(cir.p1,vector(cir.image.pos_x,cir.image.pos_y+bmap_height(cir.image.bmap)*0.5*cir.image.scale_y,0));
	//	vec_set(cir.p2,vector(cir.image.pos_x+bmap_width(cir.image.bmap)*cir.image.scale_x,cir.image.pos_y+bmap_height(cir.image.bmap)*cir.image.scale_y,0));
	//}

	for(i=0;i<cir.total_points;i++)
	{
		cir.side[i] = new(obj_side);
		if(i == 0)
		{
			vec_set(cir.side[i].start,cir.p0);
			vec_set(cir.side[i].end,cir.p1);
		}
		if(i == 1)
		{
			vec_set(cir.side[i].start,cir.p1);
			vec_set(cir.side[i].end,cir.p2);
		}

		if(i == 2)
		{
			vec_set(cir.side[i].start,cir.p2);
			vec_set(cir.side[i].end,cir.p0);
		}

		vec_diff(cir.side[i].dir,cir.side[i].end,cir.side[i].start);
		vec_set(cir.side[i].center,vector(cir.side[i].start.x+cir.side[i].dir.x*0.5,cir.side[i].start.y+cir.side[i].dir.y*0.5,0));

		//  ,   
		if(_type == cir_tile_concave1||_type == cir_tile_concave2||_type == cir_tile_concave5||_type == cir_tile_concave6)
		{
			cir.side[i].axis.x = cir.side[i].dir.y;
			cir.side[i].axis.y = cir.side[i].dir.x*-1;
		}

		if(_type == cir_tile_concave3||_type == cir_tile_concave4||_type == cir_tile_concave7||_type == cir_tile_concave8)
		{
			cir.side[i].axis.x = cir.side[i].dir.y*-1;
			cir.side[i].axis.y = cir.side[i].dir.x;
		}

		cir.side[i].axis.z = 0;
		cir.side[i].coll_axis_type = 0;

		if(cir.side[i].start.x == cir.side[i].end.x)
		{//      
			cir.side[i].coll_axis_type = coll_axis_x;
		}

		if(cir.side[i].start.y == cir.side[i].end.y)
		{//      
			cir.side[i].coll_axis_type = coll_axis_y;
		}

		if(cir.side[i].start.x != cir.side[i].end.x && cir.side[i].start.y != cir.side[i].end.y)
		{//  
			cir.side[i].coll_axis_type = coll_axis_diag;
		}

		if(cir.side[i].start.x == cir.side[i].end.x && cir.side[i].start.y == cir.side[i].end.y)
		{// 

		}

		cir.side[i].length = vec_length(cir.side[i].dir);
	}

	cir.width = bmap_width(cir.image.bmap)*cir.image.scale_x;
	cir.height = bmap_height(cir.image.bmap)*cir.image.scale_y;
	cir.type = _type;

	vec_for_pan_center(cir.center,cir.image);

	cir.c_type = c_type_circle_concave;
	if(_layer != 77){grid_tiles[_x_slot][_y_slot] = cir;}
	return(cir);
}

int check_for_collision(int _type,tile* _tile)
{
	VECTOR square_p1,square_p2,square_p3,square_p4;
	VECTOR knockback;
	var kbz,kbx,kby;
	int collided = 0;

	vec_set(square_p1.x,vector(square_pan.pos_x,square_pan.pos_y,0));
	vec_set(square_p2.x,vector(square_pan.pos_x+bmap_width(square_pan.bmap)*square_pan.scale_x,square_pan.pos_y,0));
	vec_set(square_p3.x,vector(square_pan.pos_x+bmap_width(square_pan.bmap)*square_pan.scale_x,square_pan.pos_y+bmap_height(square_pan.bmap)*square_pan.scale_y,0));
	vec_set(square_p4.x,vector(square_pan.pos_x,square_pan.pos_y+bmap_height(square_pan.bmap)*square_pan.scale_y,0));

	kb.x = 0;
	kb.y = 0;

	if(_type == c_type_triangle)
	{
		kbz = proj_intersect_diagonal(square_p1,square_p2,square_p3,square_p4,_tile,2,1,knockback);
		kbx = proj_intersect_hori(square_p1,square_p2,_tile,2);
		kby = proj_intersect_vert(square_p1,square_p4,_tile,0);
				
		if(kbx&&kby&&kbz)
		{
			collided = 1;
			if(abs(kbz)<abs(kbx)&&abs(kbz)<abs(kby))
			{//  
				kb.x = knockback.x;
				kb.y = knockback.y;
			}
			else
			{
				if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
				else{kb.y = kby;}// 
			}
		}
	}

	if(_type == c_type_square)
	{
		kbx = proj_intersect_hori(square_p1,square_p2,_tile,0);
		kby = proj_intersect_vert(square_p1,square_p4,_tile,1);
		//square.pos_x += kbx;
		if(kbx&&kby)
		{
			collided = 1;
			if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
			else{kb.y = kby;}// 
		}
	}

	if(_type == c_type_circle_convex)
	{


		kbx = proj_intersect_hori(square_p1,square_p2,_tile,2);
		//if(kbx)
		//{
			kby = proj_intersect_vert(square_p1,square_p4,_tile,0);
		//}

		//if(kbx&&kby)
		//{
			kbz = proj_intersect_circle_convex(square_p1,square_p2,square_p3,square_p4,_tile,knockback);
		//}
		if(kbz!=-777)
		{
			if(kbx&&kby&&kbz)
			{
				collided = 1;
				if(abs(kbz)<abs(kbx)&&abs(kbz)<abs(kby))
				{//  
					kb.x = knockback.x;
					kb.y = knockback.y;
				}
				else
				{
					if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
					else{kb.y = kby;}// 
				}
			}
		}
		else
		{
			if(kbx&&kby)
			{
				collided = 1;
				if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
				else{kb.y = kby;}// 
			}
		}
	}

	if(_type == c_type_circle_concave)
	{
		kbx = proj_intersect_hori(square_p1,square_p2,_tile,2);
		//if(kbx)
		//{
			kby = proj_intersect_vert(square_p1,square_p4,_tile,0);
		//}

		//if(kbx&&kby)
		//{
			kbz = proj_intersect_circle_concave(square_p1,square_p2,square_p3,square_p4,_tile,knockback);
		//}

		if(kbz==-777)
		{//    ,     ,      
			if(kbx&&kby)
			{
				collided = 1;
				if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
				else{kb.y = kby;}// 
			}
		}
		else
		{
			if(kbz==-888)
			{//    

			}
			else
			{
				if(kbz==-999)
				{

				}
				else
				{
					if(kbx&&kby&&kbz)
					{
						collided = 1;
						if(abs(kbz)<abs(kbx)&&abs(kbz)<abs(kby))
						{//  
							kb.x = knockback.x;
							kb.y = knockback.y;
						}
						else
						{
							if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
							else{kb.y = kby;}// 
						}
					}
				}
			}


		//	if(kbz!=-888)
		//	{
		//		if(kbx&&kby)
		//		{
		//			collided = 1;
		//			if(abs(kbx)<abs(kby)){kb.x = kbx;}// 
		//			else{kb.y = kby;}// 
		//		}
		//	}
		}
	}

	if(collided)
	{
		square_pan.pos_x += kb.x;
		square_pan.pos_y += kb.y;
		hitside_normals.x = kb.x;
		hitside_normals.y = kb.y;
	}

	return(collided);
}

void update_tile(tile* _tile)
{
	int i;
	if(_tile.c_type == c_type_triangle)
	{
		if(_tile.type == tri_tile1)//topleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));

		}
		if(_tile.type == tri_tile2)//topright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y,0));
		}
		if(_tile.type == tri_tile3)//downright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}
		if(_tile.type == tri_tile4)//downleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}


		if(_tile.type == tri_tile5)//topleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y,0));

		}
		if(_tile.type == tri_tile6)//topright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y,0));
		}
		if(_tile.type == tri_tile7)//downright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}
		if(_tile.type == tri_tile8)//downleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}

		for(i=0;i<_tile.total_points;i++)
		{
			if(i == 0)
			{
				vec_set(_tile.side[i].start,_tile.p0);
				vec_set(_tile.side[i].end,_tile.p1);
			}

			if(i == 1)
			{
				vec_set(_tile.side[i].start,_tile.p1);
				vec_set(_tile.side[i].end,_tile.p2);
			}

			if(i == 2)
			{
				vec_set(_tile.side[i].start,_tile.p2);
				vec_set(_tile.side[i].end,_tile.p0);
			}

			vec_diff(_tile.side[i].dir,_tile.side[i].end,_tile.side[i].start);
			vec_set(_tile.side[i].center,vector(_tile.side[i].start.x+_tile.side[i].dir.x*0.5,_tile.side[i].start.y+_tile.side[i].dir.y*0.5,0));
		}
	}

	if(_tile.c_type == c_type_circle_convex)
	{
		//p0  
		if(_tile.type == cir_tile_convex1)//topleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));

		}
		if(_tile.type == cir_tile_convex2)//topright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y,0));
		}
		if(_tile.type == cir_tile_convex3)//downright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}
		if(_tile.type == cir_tile_convex4)//downleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}


		if(_tile.type == cir_tile_convex5)//topleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y,0));

		}
		if(_tile.type == cir_tile_convex6)//topright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x,_tile.image.pos_y,0));
		}
		if(_tile.type == cir_tile_convex7)//downright
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}
		if(_tile.type == cir_tile_convex8)//downleft
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
		}

		for(i=0;i<_tile.total_points;i++)
		{
			if(i == 0)
			{
				vec_set(_tile.side[i].start,_tile.p0);
				vec_set(_tile.side[i].end,_tile.p1);
			}
			if(i == 1)
			{
				vec_set(_tile.side[i].start,_tile.p1);
				vec_set(_tile.side[i].end,_tile.p2);
			}

			if(i == 2)
			{
				vec_set(_tile.side[i].start,_tile.p2);
				vec_set(_tile.side[i].end,_tile.p0);
			}

			vec_diff(_tile.side[i].dir,_tile.side[i].end,_tile.side[i].start);
			vec_set(_tile.side[i].center,vector(_tile.side[i].start.x+_tile.side[i].dir.x*0.5,_tile.side[i].start.y+_tile.side[i].dir.y*0.5,0));
		}
	}

	if(_tile.c_type == c_type_square)
	{

		if(_tile.type == square_tile1)//left
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p3,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			_tile.width = bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x;
			_tile.height = bmap_height(_tile.image.bmap)*_tile.image.scale_y;
		}

		if(_tile.type == square_tile2)//top
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p3,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			_tile.width = bmap_width(_tile.image.bmap)*_tile.image.scale_x;
			_tile.height = bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y;
		}

		if(_tile.type == square_tile3)//right
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p3,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			_tile.width = bmap_width(_tile.image.bmap)*0.5*_tile.image.scale_x;
			_tile.height = bmap_height(_tile.image.bmap)*_tile.image.scale_y;
		}

		if(_tile.type == square_tile4)//down
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p3,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			_tile.width = bmap_width(_tile.image.bmap)*_tile.image.scale_x;
			_tile.height = bmap_height(_tile.image.bmap)*0.5*_tile.image.scale_y;
		}

		if(_tile.type == square_tile5)//full
		{
			vec_set(_tile.p0,vector(_tile.image.pos_x,_tile.image.pos_y,0));
			vec_set(_tile.p1,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y,0));
			vec_set(_tile.p2,vector(_tile.image.pos_x+bmap_width(_tile.image.bmap)*_tile.image.scale_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			vec_set(_tile.p3,vector(_tile.image.pos_x,_tile.image.pos_y+bmap_height(_tile.image.bmap)*_tile.image.scale_y,0));
			_tile.width = bmap_width(_tile.image.bmap)*_tile.image.scale_x;
			_tile.height = bmap_height(_tile.image.bmap)*_tile.image.scale_y;
		}

		vec_set(_tile.center,vector(_tile.p0.x+_tile.width*0.5,_tile.p0.y+_tile.height*0.5,0));

		for(i=0;i<_tile.total_points;i++)
		{
			if(i == 0)
			{
				vec_set(_tile.side[i].start,_tile.p0);
				vec_set(_tile.side[i].end,_tile.p1);
			}
			if(i == 1)
			{
				vec_set(_tile.side[i].start,_tile.p1);
				vec_set(_tile.side[i].end,_tile.p2);
			}

			if(i == 2)
			{
				vec_set(_tile.side[i].start,_tile.p2);
				vec_set(_tile.side[i].end,_tile.p3);
			}

			if(i == 3)
			{
				vec_set(_tile.side[i].start,_tile.p3);
				vec_set(_tile.side[i].end,_tile.p0);
			}

			vec_diff(_tile.side[i].dir,_tile.side[i].end,_tile.side[i].start);
			vec_set(_tile.side[i].center,vector(_tile.side[i].start.x+_tile.side[i].dir.x*0.5,_tile.side[i].start.y+_tile.side[i].dir.y*0.5,0));
		}
	}

	//draw_line_vec2vec(_tile.p0,_tile.p1,c_green,100);
	//draw_line_vec2vec(_tile.p1,_tile.p2,c_green,100);
	//draw_line_vec2vec(_tile.p2,_tile.p3,c_green,100);
	//draw_line_vec2vec(_tile.p3,_tile.p0,c_green,100);
}

void sample()
{
	square_pan = pan_create2("button(0, 0, square.tga, square.tga, square.tga, dragmee,0,0);",10,"square.tga",450,300,VISIBLE | TRANSLUCENT | LIGHT);
	vec_set(square_pan.blue,c_blue);
	square_pan.alpha = 10;
	kb.z=0;

	VECTOR square_center,triangle_center;


	VECTOR triangle_normal;
	VECTOR hypo,normal_dir,triangle_side,proj,proj1,proj2,proj3,proj4;
	VECTOR tempvecx,tempvecy,tempvec;


	VECTOR triangle_axis_dot,cross_dot,proj_axis,knockback_dir;

	int i;
	create_tile_square("square.tga",1,1,10,square_tile5);
	create_tile_circle_concave("circle_concave_1.tga",2,1,10,cir_tile_concave1);
	create_tile_circle_concave("circle_concave_1.tga",1,2,10,cir_tile_concave1);

	create_tile_square("square.tga",1,14,10,square_tile5);
	create_tile_square("square.tga",1,15,10,square_tile5);
	create_tile_square("square.tga",2,15,10,square_tile5);
	create_tile_circle_concave("circle_concave_4.tga",1,13,10,cir_tile_concave4);
	create_tile_circle_concave("circle_concave_4.tga",3,15,10,cir_tile_concave4);
	create_tile_circle_convex("circle4.tga",2,14,10,cir_tile_convex4);

	create_tile_circle_concave("circle_concave_2.tga",3,13,10,cir_tile_concave2);
	create_tile_circle_convex("circle2.tga",2,12,10,cir_tile_convex2);
	create_tile_circle_convex("circle2.tga",4,14,10,cir_tile_convex2);
	create_tile_circle_convex("circle3.tga",2,11,10,cir_tile_convex3);
	create_tile_circle_convex("circle1.tga",5,14,10,cir_tile_convex1);
	create_tile_circle_convex("circle1.tga",3,11,10,cir_tile_convex1);
	create_tile_circle_convex("circle4.tga",3,12,10,cir_tile_convex4);
	create_tile_circle_concave("circle_concave_4.tga",4,13,10,cir_tile_concave4);

	create_tile_square("square.tga",3,10,10,square_tile5);
	create_tile_square("square.tga",3,9,10,square_tile5);
	create_tile_square("square.tga",3,8,10,square_tile5);
	create_tile_square("square.tga",3,7,10,square_tile5);
	create_tile_square("square.tga",3,6,10,square_tile5);


	create_tile_circle_concave("circle_concave_3.tga",3,5,10,cir_tile_concave3);
	create_tile_circle_concave("circle_concave_1.tga",4,5,10,cir_tile_concave1);
	create_tile_circle_concave("circle_concave_3.tga",4,4,10,cir_tile_concave3);
	create_tile_circle_concave("circle_concave_1.tga",5,4,10,cir_tile_concave1);


	create_tile_circle_concave("circle_concave_3.tga",5,3,10,cir_tile_concave3);
	create_tile_circle_concave("circle_concave_1.tga",6,3,10,cir_tile_concave1);
	create_tile_circle_concave("circle_concave_3.tga",6,2,10,cir_tile_concave3);
	create_tile_circle_concave("circle_concave_1.tga",7,2,10,cir_tile_concave1);


	create_tile_circle_convex("circle3.tga",6,11,10,cir_tile_convex3);
	create_tile_circle_convex("circle4.tga",7,11,10,cir_tile_convex4);
	create_tile_circle_convex("circle2.tga",6,12,10,cir_tile_convex2);
	create_tile_circle_convex("circle1.tga",7,12,10,cir_tile_convex1);

	create_tile_circle_convex("circle4.tga",1,8,10,cir_tile_convex4);
	create_tile_circle_convex("circle1.tga",1,9,10,cir_tile_convex1);

	create_tile_square("square.tga",2,3,10,square_tile5);

	create_tile_square("square.tga",13,1,10,square_tile5);
	create_tile_square("square.tga",13,2,10,square_tile5);
	create_tile_square("square.tga",13,3,10,square_tile5);

	create_tile_square("square.tga",14,1,10,square_tile5);
	create_tile_square("square.tga",14,2,10,square_tile5);
	create_tile_square("square.tga",14,3,10,square_tile5);

	create_tile_circle_convex("circle3.tga",9,10,10,cir_tile_convex3);
	create_tile_circle_convex("circle4.tga",11,10,10,cir_tile_convex4);
	create_tile_circle_convex("circle2.tga",9,11,10,cir_tile_convex2);
	create_tile_circle_convex("circle1.tga",11,11,10,cir_tile_convex1);

	create_tile_square("square.tga",10,10,10,square_tile5);
	create_tile_square("square.tga",10,11,10,square_tile5);

	create_tile_square("square.tga",11,1,10,square_tile5);
	create_tile_square("square.tga",12,1,10,square_tile5);
	create_tile_square("square.tga",12,2,10,square_tile5);


	create_tile_triangle("tri2.tga",10,1,10,tri_tile2);
	create_tile_triangle("tri2.tga",11,2,10,tri_tile2);
	create_tile_triangle("tri2.tga",12,3,10,tri_tile2);

	create_tile_triangle("tri1.tga",3,3,10,tri_tile1);

	create_tile_triangle("tri6.tga",13,4,10,tri_tile6);
	create_tile_square("square.tga",14,4,10,square_tile5);

	create_tile_circle_convex("circle2.tga",14,5,10,cir_tile_convex2);
	create_tile_circle_convex("circle1.tga",15,5,10,cir_tile_convex1);
	create_tile_circle_convex("circle4.tga",15,4,10,cir_tile_convex4);

	create_tile_triangle("tri5.tga",6,9,10,tri_tile5);
	create_tile_triangle("tri6.tga",5,9,10,tri_tile6);
	create_tile_triangle("tri7.tga",5,8,10,tri_tile7);
	create_tile_triangle("tri8.tga",6,8,10,tri_tile8);

	create_tile_triangle("tri7.tga",13,15,10,tri_tile7);

	player_tiles[0] = create_tile_circle_convex("circle3.tga",21,5,77,cir_tile_convex3);
	player_tiles[1] = create_tile_square("square.tga",21,6,77,square_tile5);
	player_tiles[2] = create_tile_square("square.tga",21,7,77,square_tile5);
	player_tiles[3] = create_tile_square("square.tga",21,8,77,square_tile5);
	player_tiles[4] = create_tile_circle_convex("circle2.tga",21,9,77,cir_tile_convex2);

	var	player_top_dot_x = player_tiles[0].image.pos_x;
	var player_top_dot_y = player_tiles[0].image.pos_y;

	for(i=0;i<17;i++)
	{
		create_tile_square("square.tga",0,i,10,square_tile5);
	}

	for(i=0;i<17;i++)
	{
		create_tile_square("square.tga",23,i,10,square_tile5);
	}

	for(i=1;i<23;i++)
	{
		create_tile_square("square.tga",i,0,10,square_tile5);
	}

	for(i=1;i<23;i++)
	{
		create_tile_square("square.tga",i,16,10,square_tile5);
	}

	//vec_set(square_p1.x,vector(square.pos_x,square.pos_y,0));
	//vec_set(square_p2.x,vector(square.pos_x+bmap_width(square.bmap)*square.scale_x,square.pos_y,0));
	//vec_set(square_p3.x,vector(square.pos_x+bmap_width(square.bmap)*square.scale_x,square.pos_y+bmap_height(square.bmap)*square.scale_y,0));
	//vec_set(square_p4.x,vector(square.pos_x,square.pos_y+bmap_height(square.bmap)*square.scale_y,0));

	VECTOR dir,new_dir;
	dir.x = -1;
	dir.y = 1;
	dir.z = 0;

	new_dir.x = 0;
	new_dir.y = 0;
	new_dir.z = 0;

	var dp = 0;
	var cres = 0;
	var collided = 0;
	var player_speed = 0;
	while(1)
	{
		if(key_r)
		{
			square_pan.pos_x = 450;
			square_pan.pos_y = 300;
			dir.x = -1;
			dir.y = 1;
			dir.z = 0;
		}

		collided = 0;
		//vec_for_pan_center(square_center,square);
		for(i=1;i<17;i++)
		{
			draw_line_vec2vec(vector(0,i*40,0),vector(960,i*40,0),c_white,10);
		}

		for(i=1;i<24;i++)
		{
			draw_line_vec2vec(vector(i*40,0,0),vector(i*40,680,0),c_white,10);
		}

		for(i=0;i<x_slots*y_slots;i++)
		{
			if(grid_tiles[0][i]!=0)
			{
				cres = check_for_collision(grid_tiles[0][i].c_type,grid_tiles[0][i]);
				if(collided < cres){collided = cres;}
			}
		}

		
		//draw_line_vec2vec(vector(player_top_dot_x,player_top_dot_y,0),vector(mouse_pos.x,mouse_pos.y,0),c_blue,100);

		player_speed = time_step*abs(player_top_dot_y - mouse_pos.y);
		player_speed = minv(player_speed,10);

		if(player_top_dot_y < mouse_pos.y){player_top_dot_y += player_speed;}
		if(player_top_dot_y > mouse_pos.y){player_top_dot_y -= player_speed;}
		player_top_dot_y = clamp(player_top_dot_y,40,640-200);


		for(i=0;i<player_size;i++)
		{
			player_tiles[i].image.pos_y = player_top_dot_y+i*40;
		}

		for(i=0;i<player_size;i++)
		{
			if(player_tiles[i] != 0)
			{
				update_tile(player_tiles[i]);
			}
		}
		
		for(i=0;i<player_size;i++)
		{
			cres = check_for_collision(player_tiles[i].c_type,player_tiles[i]);
			if(collided < cres){collided = cres;}
		}


		//speed = 0;

		if(mouse_right)
		{
			speed = time_step*abs(square_pan.pos_x - mouse_pos.x);

			if(speed < 2){draw_text(str_for_num(0,speed),10,10, c_green);}
			else{draw_text(str_for_num(0,speed),10,10, c_red);}
			
			speed = minv(speed,2);

			if(square_pan.pos_x < mouse_pos.x){square_pan.pos_x += speed;}
			if(square_pan.pos_x > mouse_pos.x){square_pan.pos_x -= speed;}


			speed = time_step*abs(square_pan.pos_y - mouse_pos.y);

			if(speed < 2){draw_text(str_for_num(0,speed),10,30, c_green);}
			else{draw_text(str_for_num(0,speed),10,30, c_red);}

			speed = minv(speed,2);

			if(square_pan.pos_y < mouse_pos.y){square_pan.pos_y += speed;}
			if(square_pan.pos_y > mouse_pos.y){square_pan.pos_y -= speed;}
			//speed = 0;
		}
		else
		{
			if(speed)
			{
				if(collided == 1)
				{
					vec_normalize(hitside_normals,1);
					dp = vec_dot(vector(dir.x,dir.y,0),vector(hitside_normals.x,hitside_normals.y,0));
					dir.x = myspeed_mod * (-2 * dp * hitside_normals.x + dir.x);
					dir.y = myspeed_mod * (-2 * dp * hitside_normals.y + dir.y);
				}
				
				if(dir.y < 1){dir.y += mygravity*time_step;}
				
				square_pan.pos_x += dir.x*speed*time_step;
				square_pan.pos_y += dir.y*speed*time_step;
			}
		}

		//draw_text(str_for_num(0,dir.x),300,50, c_green);
		//draw_text(str_for_num(0,dir.y),300,70, c_green);
		//draw_text(str_for_num(0,hitside_normals.x),300,90, c_green);
		//draw_text(str_for_num(0,hitside_normals.y),300,110, c_green);

		draw_text("drag and drop blue box with left mouse",100,10, c_blue);
		draw_text("make it follow cursor by holding right mouse",100,30, c_blue);
		draw_text("make it fly or stop with space",100,50, c_blue);
		draw_text("hit R if box got off screen",100,70, c_blue);

		//draw_text(str_for_num(0,mouse_pos.x/40),500,20, c_green);
		//draw_text(str_for_num(0,mouse_pos.y/40),500,50, c_green);

		//draw_text("y+",770,50, c_green);
		//draw_text("bounce %",800,50, c_green);
		//draw_text(str_for_num(0,mybounce),890,50, c_green);
		//draw_text("u-",930,50, c_green);

		//draw_text("h+",770,80, c_green);
		//draw_text("friction %",800,80, c_green);
		//draw_text(str_for_num(0,myfriction),890,80, c_green);
		//draw_text("j-",930,80, c_green);

		//draw_text("n+",770,110, c_green);
		//draw_text("gravity %",800,110, c_green);
		//draw_text(str_for_num(0,mygravity),890,110, c_green);
		//draw_text("m-",930,110, c_green);

		//draw_text("i+",770,140, c_green);
		//draw_text("speed_mod %",800,140, c_green);
		//draw_text(str_for_num(0,myspeed_mod),930,140, c_green);
		//draw_text("k-",970,140, c_green);

		wait(1);
	}
}



void main()
{
	wait(3);
	level_load(0);
	video_set(960,680,0,0);
	vec_set(sky_color,vector(160,160,160));
	wait(3);
	mouse_pointer = 0;
	mouse_mode = 4;

	//on_y = bounceup;
	//on_u = bouncedown;
	//on_h = frictionup;
	//on_j = frictiondown;
	//on_n = gravityup;
	//on_m = gravitydown;
	//on_i = speedup;
	//on_k = speeddown;

	on_space = speedtg;

	mouse_map = bmap_create("mouse.tga");

	int i,j;
	for(i=0;i<x_slots;i++)
	{
		for(j=0;j<y_slots;j++)
		{
			grid_tiles[i][j] = 0;
		}	
	}

	sample();
}
